package org.example.wedding_app_backend.api.controller;

import org.example.wedding_app_backend.wedding.controller.WeddingController;
import org.example.wedding_app_backend.wedding.dto.GetWeddingResponse;
import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.function.WeddingMapper;
import org.example.wedding_app_backend.wedding.service.WeddingService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.result.MockMvcResultHandlers;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;

import java.util.Optional;
import java.util.UUID;

import static org.hamcrest.CoreMatchers.is;
import static org.mockito.BDDMockito.when;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;

@WebMvcTest(controllers = WeddingController.class)
@AutoConfigureMockMvc(addFilters = false)
public class WeddingControllerTests {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private WeddingService weddingService;

    @MockBean
    private WeddingMapper weddingMapper;

    private WeddingEntity weddingEntity;

    private GetWeddingResponse getWeddingResponse;

    @BeforeEach
    void setUp() {
        UUID weddingId = UUID.randomUUID();
        weddingEntity = mock(WeddingEntity.class);
        when(weddingEntity.getId()).thenReturn(weddingId);
        getWeddingResponse = GetWeddingResponse.builder().id(weddingId).build();
    }

    @Test
    void getWedding_ShouldReturnWedding_WhenExists() throws Exception {
        when(weddingService.findWedding(any())).thenReturn(Optional.of(weddingEntity));
        when(weddingMapper.entityToRepresentation(any())).thenReturn(getWeddingResponse);

        ResultActions response = mockMvc.perform(get("/api/wedding/{id}", weddingEntity.getId())
                .contentType(MediaType.APPLICATION_JSON));

        response.andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(MockMvcResultMatchers.jsonPath("$.id", is(weddingEntity.getId().toString())))
                .andDo(MockMvcResultHandlers.print());
    }

    @Test
    void getWedding_ShouldReturnNotFound_WhenDoesNotExist() throws Exception {
        when(weddingService.findWedding(any())).thenReturn(Optional.empty());

        ResultActions response = mockMvc.perform(get("/api/wedding/{id}", weddingEntity.getId())
                .contentType(MediaType.APPLICATION_JSON));

        response.andExpect(MockMvcResultMatchers.status().isNotFound());
    }

    @Test
    void getWeddingImage_ShouldReturnImage_WhenExists() throws Exception {
        byte[] image = new byte[]{1, 2, 3};
        when(weddingService.findWedding(weddingEntity.getId())).thenReturn(Optional.of(weddingEntity));
        when(weddingEntity.getBackgroundImage()).thenReturn(image);

        ResultActions response = mockMvc.perform(get("/api/wedding/{weddingId}/background", weddingEntity.getId())
                .contentType(MediaType.IMAGE_JPEG));

        response.andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(MockMvcResultMatchers.content().bytes(image));
    }

    @Test
    void getWeddingImage_ShouldReturnNotFound_WhenImageDoesNotExist() throws Exception {
        when(weddingService.findWedding(weddingEntity.getId())).thenReturn(Optional.of(weddingEntity));
        when(weddingEntity.getBackgroundImage()).thenReturn(null);

        ResultActions response = mockMvc.perform(get("/api/wedding/{weddingId}/background", weddingEntity.getId())
                .contentType(MediaType.IMAGE_JPEG));

        response.andExpect(MockMvcResultMatchers.status().isNotFound());
    }

    @Test
    void putWeddingImage_ShouldSaveImage() throws Exception {
        MockMultipartFile mockFile = new MockMultipartFile("image", "background.jpg", "image/jpeg", new byte[]{1, 2, 3});

        ResultActions response = mockMvc.perform(multipart("/api/wedding/{weddingId}/background", weddingEntity.getId())
                .file(mockFile)
                .contentType(MediaType.MULTIPART_FORM_DATA));

        response.andExpect(MockMvcResultMatchers.status().isCreated());
    }
}

