import {
  Component,
  HostListener, Input,
  OnInit,
  signal,
  WritableSignal
} from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { GalleryService } from '../../services/gallery.service';
import { NgClass, NgForOf, NgIf } from '@angular/common';
import { CascadeSelectModule } from 'primeng/cascadeselect';
import { PanelModule } from 'primeng/panel';
import { FileUploadModule } from 'primeng/fileupload';
import { ButtonGroupModule } from 'primeng/buttongroup';
import { catchError } from 'rxjs';
import {ImageType} from '../../models/image.type';

@Component({
  selector: 'app-gallery',
  standalone: true,
  imports: [
    NgIf,
    NgForOf,
    CascadeSelectModule,
    PanelModule,
    NgClass,
    FileUploadModule,
    ButtonGroupModule
  ],
  templateUrl: './gallery.component.html',
  styleUrls: ['./gallery.component.css']
})
export class GalleryComponent implements OnInit {

  gallery: [ImageType] = [{id: '', likes: 0, date: '', liked: false}];

  selectedFile: File | null = null;

  previewUrl: string | ArrayBuffer | null = null;

  @Input()
  weddingId: string = '';

  @Input()
  guestId: string = '';

  activeSort: string = 'date';

  constructor(private route: ActivatedRoute,
              private galleryService: GalleryService) {
  }

  ngOnInit() {
    this.loadGallery();
  }

  onDownloadImageAction(imageId: string): void {
    const url = `/api/wedding/${this.weddingId}/gallery/${imageId}`;
    const link = document.createElement('a');
    link.href = url;
    link.download = `${imageId}.jpg`;
    link.style.display = 'none';

    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  }

  onTriggerFileInputAction(): void {
    const fileInput = document.getElementById('fileInput') as HTMLInputElement;
    fileInput?.click();
  }

  onFileSelected(event: Event): void {
    const input = event.target as HTMLInputElement;
    if (input.files && input.files.length > 0) {
      this.selectedFile = input.files[0];

      const reader = new FileReader();
      reader.onload = () => (this.previewUrl = reader.result);
      reader.readAsDataURL(this.selectedFile);

      const container = document.getElementById('show-selected-image-container');
      if (container) {
        container.style.display = 'flex';
      }
    }
  }

  onSendButtonAction(): void {
    if (this.selectedFile) {
      this.galleryService.postImage(this.selectedFile, this.weddingId, this.guestId).subscribe({
        next: () => this.loadGallery(),
        error: (err) => console.error('Error uploading image:', err)
      });
      this.onImageClearAction();
    }
  }

  onImageClearAction(): void {
    const container = document.getElementById('show-selected-image-container');
    if (container) {
      container.style.display = 'none';
    }
    this.selectedFile = null;
    this.previewUrl = null;
  }

  onSortButtonAction(sortType: string): void {
    this.activeSort = sortType;
    this.loadGallery();
  }

  onLikeButtonAction(image: ImageType): void {
    image.liked = !image.liked;

    if (image.liked) {
      image.likes = image.likes + 1;
    }
    else {
      image.likes = image.likes - 1;
    }

    this.galleryService.changeImageLikes(this.weddingId, this.guestId, image.id, image.liked).subscribe({
      next: (response) => console.log('Like updated successfully:', response),
      error: (error) => console.error('Error updating like:', error)
    });
  }

  loadGallery(): void {
    this.galleryService
      .getGalleryByWeddingId(this.weddingId, this.guestId, this.activeSort)
      .pipe(catchError((error) => {
        console.error(error);
        throw error;
      }))
      .subscribe((weddingGallery) => (this.gallery = weddingGallery));
  }

}
